// Application : jhnSetup
// jhnIniFile.prg , Created : 2009/06/24   04:43 PM

#using System.Collections
#using System.Collections.Generic
#using System.Text
#using System.Xml
#using System.IO

BEGIN NAMESPACE jhnFT.Utils.Config

	CLASS jhnIniFile
		PROTECT sFile AS STRING // Name of the INI file
		PROTECT aSections AS SortedList<STRING, OBJECT>

		CONSTRUCTOR()
			SELF:InitializeIni("")
		RETURN

		CONSTRUCTOR(cF AS STRING)
			LOCAL cExt AS STRING
			SELF:InitializeIni(cF)
			IF (cExt := Path.GetExtension(cF):ToLower()):Contains(".ini")
				SELF:ReadIni()
			ELSEIF cExt:Contains(".config")
				SELF:ReadXml()
			ENDIF
		RETURN

		HIDDEN METHOD InitializeIni(cF AS STRING) AS VOID
			SELF:sFile := cF
			SELF:aSections := SortedList<STRING, OBJECT>{}
		RETURN

		ACCESS FileName AS STRING
			RETURN SELF:sFile

		ASSIGN FileName(cF AS STRING) AS VOID
			SELF:sFile := cF

		METHOD Clear() AS VOID
			SELF:aSections := SortedList<STRING, OBJECT>{}
		RETURN

		METHOD GetSection( sSection AS STRING ) AS STRING[]
			LOCAL sValue AS STRING[]
			LOCAL oSectionItems AS jhnSectionItems
			LOCAL o AS OBJECT
			IF SELF:aSections:TryGetValue(sSection:ToLower(), o)
				oSectionItems := (jhnSectionItems)o
				sValue := oSectionItems:GetItems()
			ELSE
				sValue := STRING[]{0}
			ENDIF
		RETURN sValue

		METHOD GetItemNames( sSection AS STRING ) AS STRING[]
			LOCAL sValue AS STRING[]
			LOCAL oSectionItems AS jhnSectionItems
			LOCAL o AS OBJECT
			LOCAL i AS INT
			IF SELF:aSections:TryGetValue(sSection:ToLower(), o)
				oSectionItems := (jhnSectionItems)o
				sValue := STRING[]{oSectionItems:GetItems():Length}
				FOR i := 0 UPTO sValue:Length - 1
					sValue[i] := oSectionItems:GetItems()[i]:Substring(0, oSectionItems:GetItems()[i]:IndexOf("="))
				NEXT
			ELSE
				sValue := STRING[]{0}
			ENDIF
		RETURN sValue

		METHOD GetSectionNames() AS STRING[]
			LOCAL sValue AS STRING[]
			LOCAL nSection AS INT
			sValue := STRING[]{SELF:aSections:Count}
			FOR nSection := 0 UPTO sValue:Length - 1
				sValue[nSection] := SELF:aSections:Keys[nSection]
			NEXT
		RETURN sValue

		METHOD GetString(sSection AS STRING, sEntry AS STRING) AS STRING
			LOCAL sValue AS STRING	// Value for the entry
			LOCAL o AS OBJECT
			LOCAL oSection AS jhnSectionItems
			IF SELF:aSections:TryGetValue(sSection:ToLower(), o)
				oSection := (jhnSectionItems)o
				sValue := oSection:GetItem(sEntry):Trim()
			ELSE
				sValue := ""
			ENDIF
		RETURN sValue

		METHOD GetDWord(sSection AS STRING, sEntry AS STRING) AS DWORD
			LOCAL sValue AS STRING
			LOCAL nValue AS DWORD
			sValue := SELF:GetString(sSection, sEntry)
			TRY
				nValue := UInt32.Parse(sValue)
			CATCH
				nValue := 0
			END TRY
		RETURN nValue

		METHOD GetInt(sSection AS STRING, sEntry AS STRING) AS INT
			LOCAL sValue AS STRING
			LOCAL nValue AS INT
			sValue := SELF:GetString(sSection, sEntry)
			TRY
				nValue := Int32.Parse(sValue)
			CATCH
				nValue := 0
			END TRY
		RETURN nValue

		METHOD GetLogic(sSection AS STRING, sEntry AS STRING) AS LOGIC
			LOCAL sValue AS STRING
			LOCAL lValue AS LOGIC
			sValue := SELF:GetString(sSection, sEntry):ToUpper()
			lValue := "_TRUE_YES_ON_":Contains("_" + sValue + "_")
		RETURN lValue

		METHOD GetStringUpper(sSection AS STRING, sEntry AS STRING) AS STRING
			LOCAL sValue AS STRING
			sValue := SELF:GetString(sSection, sEntry)
		RETURN sValue:ToUpper()

		METHOD SetString(sSection AS STRING, sEntry AS STRING, sString AS STRING) AS LOGIC
			LOCAL o AS OBJECT
			LOCAL oSection AS jhnSectionItems
			IF SELF:aSections:TryGetValue(sSection:ToLower(), o)
				oSection := (jhnSectionItems)o
				oSection:PutItem(sEntry, sString)
			ELSE
				SELF:aSections:Add(sSection:ToLower(), oSection := jhnSectionItems{})
				oSection:PutItem(sEntry:ToLower(), sString)
			ENDIF
			// Not sure why SetString can fail, hence the return true
		RETURN TRUE

		METHOD SetInt(sSection AS STRING, sEntry AS STRING, nInt AS INT) AS LOGIC
			LOCAL sInt AS STRING
			sInt := nInt:ToString()
			RETURN SELF:SetString(sSection, sEntry, sInt)

		METHOD SetDWord(sSection AS STRING, sEntry AS STRING, nInt AS DWORD) AS LOGIC
			LOCAL sInt AS STRING
			sInt := nInt:ToString()
			RETURN SELF:SetString(sSection, sEntry, sInt)

		METHOD SetLogic( sSection AS STRING, sEntry AS STRING, lValue AS LOGIC ) AS LOGIC
			LOCAL sValue AS STRING
			sValue := lValue:ToString()
		RETURN SELF:SetString(sSection, sEntry, sValue)

		METHOD DeleteEntry( sSection AS STRING, sEntry AS STRING ) AS LOGIC
		RETURN SELF:SetString( sSection, sEntry, "" )

		METHOD DeleteSection( sSection AS STRING ) AS LOGIC
			LOCAL o AS OBJECT
			IF SELF:aSections:TryGetValue(sSection:ToLower(), o)
				SELF:aSections:Remove(sSection:ToLower())
				SELF:aSections:TrimExcess()
			ENDIF
		RETURN TRUE

		METHOD IsSection(sSection AS STRING) AS LOGIC
			LOCAL nIndex AS INT
			nIndex := SELF:aSections:IndexOfKey(sSection:ToLower())
			RETURN nIndex >= 0

		METHOD IsEntry(sSection AS STRING, sEntry AS STRING) AS LOGIC
			LOCAL oSec AS jhnSectionItems
			LOCAL lIsEntry AS LOGIC
			IF (lIsEntry := SELF:IsSection(sSection := sSection:ToLower()))
				oSec := (jhnSectionItems)SELF:aSections:Item[sSection]
				lIsEntry := oSec:IsEntry(sEntry)
			ENDIF
		RETURN lIsEntry

		METHOD FlushIni() AS VOID
			LOCAL sr AS StreamWriter
			LOCAL sb AS StringBuilder
			LOCAL s, i, sc, ic AS INT
			LOCAL sn, its AS STRING[]
			LOCAL sFileIni AS STRING

			sc := SELF:aSections:Count
			sn := SELF:GetSectionNames()
			sb := StringBuilder{}

			FOR s := 0 UPTO sc - 1
				sb:AppendLine("[" + sn[s] + "]")
				its := SELF:GetSection(sn[s])
				ic := its:Length
				FOR i := 0 UPTO ic - 1
					sb:AppendLine(its[i])
				NEXT
				sb:AppendLine()
			NEXT

			sFileIni := SELF:sFile:Replace(Path.GetExtension(SELF:sFile), ".ini")
			//Lets make a copy of the current file, just in case we made a mistake
			IF File.Exists(sFileIni)
				File.Copy(sFileIni, sFileIni + ".bak", TRUE)
			ENDIF
			sr := StreamWriter{sFileIni, FALSE, Encoding.ASCII}
			sr:Write(sb:ToString())
			sr:Close()
			sr:Dispose()
		RETURN

		METHOD FlushXml() AS VOID
			LOCAL sFileXml AS STRING
			LOCAL xw AS XmlTextWriter
			LOCAL s, n, v AS STRING
			LOCAL sSections, sItems AS STRING[]
			LOCAL i, j AS INT

			sFileXml := SELF:sFile:Replace(Path.GetExtension(SELF:sFile), ".config")
			IF File.Exists(sFileXml)
				File.Copy(sFileXml, sFileXml + ".bak", TRUE)
			ENDIF

			xw := XmlTextWriter{sFileXml, Encoding.UTF8}
			xw:Formatting := Formatting.Indented
			xw:WriteStartDocument()
			xw:WriteStartElement("configuration")

			sSections := SELF:GetSectionNames()
			FOR i := 0 UPTO sSections:Length - 1
				s := sSections[i]
				xw:WriteStartElement("section")
				xw:WriteAttributeString("name", s)
				sItems := SELF:GetItemNames(s)
				FOR j := 0 UPTO sItems:Length - 1
					n := sItems[j]
					v := SELF:GetString(s, n)
					xw:WriteStartElement("entry")
					xw:WriteAttributeString("name", n)
					xw:WriteAttributeString("value", v)
					xw:WriteEndElement()
				NEXT
				xw:WriteEndElement()
			NEXT

			xw:WriteEndDocument()

			xw:Close()

		RETURN

		METHOD ReadIni() AS VOID
			LOCAL sFileIni AS STRING
			LOCAL oSR AS StreamReader
			LOCAL sBuild AS StringBuilder
			LOCAL sTemp, sSectionName, sSectionBody, sItemName, sItemValue AS STRING
			LOCAL sSection, sBodySplit, sSectionItems, sItem AS STRING[]
			LOCAL nSection, nBody, nItem, nValue AS INT

			sFileIni := SELF:sFile:Replace(Path.GetExtension(SELF:sFile), ".ini")
			IF File.Exists(sFileIni)
				oSR := StreamReader{sFileIni, TRUE}
				sBuild := StringBuilder{}
				WHILE (sTemp := oSR:ReadLine()) <> NULL
					sBuild:AppendLine(sTemp)
				ENDDO
				oSR:Close()
				oSR:Dispose()
				sTemp := sBuild:ToString()
				sSection := sTemp:Split("[":ToCharArray(), StringSplitOptions.RemoveEmptyEntries)
				FOR nSection := 0 UPTO sSection:Length - 1
					sTemp := sSection[nSection]
					sBodySplit := sTemp:Split("]":ToCharArray(), StringSplitOptions.RemoveEmptyEntries)
					sSectionName := sBodySplit[0]:ToLower()
					sSectionBody := ""
					IF sBodySplit:Length > 1
						FOR nBody := 1 UPTO sBodySplit:Length - 1
							sSectionBody += sBodySplit[nBody]
						NEXT
					ENDIF
					sSectionItems := sSectionBody:Split((Chr(13) + Chr(10)):ToCharArray(), StringSplitOptions.RemoveEmptyEntries)
					FOR nItem := 0 UPTO sSectionItems:Length - 1
						sItem := sSectionItems[nItem]:Split("=":ToCharArray(), StringSplitOptions.RemoveEmptyEntries)
						sItemName := sItem[0]
						sItemValue := ""
						FOR nValue := 1 UPTO sItem:Length - 1
							sItemValue += sItem[nValue]
						NEXT
						SELF:SetString(sSectionName, sItemName, sItemValue)
					NEXT
				NEXT
			ENDIF
		RETURN

		METHOD ReadXml() AS VOID
			LOCAL sFileXml AS STRING
			LOCAL xr AS XmlReader
			LOCAL settings AS XmlReaderSettings
			LOCAL s, n, v AS STRING

			sFileXml := SELF:sFile:Replace(Path.GetExtension(SELF:sFile), ".config")
			IF !File.Exists(sFileXml)
				RETURN
			ENDIF

			settings := XmlReaderSettings{}
			settings:IgnoreWhitespace := TRUE

			xr := XmlReader.Create(sFileXml, settings)
			xr:MoveToContent()

			IF xr:Name == "configuration"
				WHILE xr:Read()
					IF xr:NodeType = XmlNodeType.EndElement
						LOOP
					ENDIF
					IF xr:Name == "section"
						s := xr["name"]
					ELSEIF xr:Name == "entry"
						n := xr["name"]
						v := xr["value"]
						SELF:SetString(s, n, v)
					ENDIF
				ENDDO
			ENDIF

			xr:Close()

		RETURN

		METHOD CommentSection(sSection AS STRING) AS VOID
			sSection := sSection:ToLower()
			IF SELF:IsSection(sSection)
				SELF:aSections:Add(";" + sSection, SELF:aSections[sSection])
				SELF:DeleteSection(sSection)
			ENDIF
		RETURN

		METHOD CommentEntry(sSection AS STRING, sEntry AS STRING) AS VOID
			LOCAL sValue AS STRING
			sSection := sSection:ToLower()
			sEntry := sEntry:ToLower()
			IF SELF:IsEntry(sSection, sEntry)
				sValue := SELF:GetString(sSection, sEntry)
				SELF:SetString(sSection, ";" + sEntry, sValue)
				SELF:DeleteEntry(sSection, sEntry)
			ENDIF
		RETURN

		METHOD UnCommentSection(sSection AS STRING) AS VOID
			sSection := sSection:ToLower()
			IF SELF:IsSection(";" + sSection)
				SELF:aSections:Add(sSection, SELF:aSections[";" + sSection])
				SELF:DeleteSection(";" + sSection)
			ENDIF
		RETURN

		METHOD UnCommentEntry(sSection AS STRING, sEntry AS STRING) AS VOID
			LOCAL sValue AS STRING
			IF SELF:IsEntry(sSection, ";" + sEntry)
				sValue := SELF:GetString(sSection, ";" + sEntry)
				SELF:SetString(sSection, sEntry, sValue)
				SELF:DeleteEntry(sSection, ";" + sEntry)
			ENDIF
		RETURN

	END CLASS

END NAMESPACE

